# this script inputs the file "randomer_genomic_weighted_match_list_with_replicate_counts", output from "experimental_randomer_counter.pl" ,
# to determine which hundred base pair bins are linked to randomers with outlying activity based on the dixon Q test.  These outlying hundred-base pair
# bin-randomer conbinations are printed to the output file "bad_matches". The "bad_matches" file must be moved to the "count_data_by_100_bp_bin" folder
# by the user


#Input
$fileIn_match = "randomer_genomic_weighted_match_list_with_replicate_counts";  # randomer match list with experimental 
				#replicate counts, output from "experimental_randomer_counter.pl"

#Output File
$file_out = "bad_matches";	# list of bad randomer-locus matches

$arrayTMT[3]= 0.994;
$arrayTMT[4]= 0.926;
$arrayTMT[5]= 0.821;
$arrayTMT[6]= 0.740;
$arrayTMT[7]= 0.68;
$arrayTMT[8]= 0.634;
$arrayTMT[9]= 0.598;
$arrayTMT[10]= 0.568;
$arrayTMT[11]= 0.542;
$arrayTMT[12]= 0.522;
$arrayTMT[13]= 0.503;
$arrayTMT[14]= 0.488;			#these are the dixon values for the outlier test
$arrayTMT[15]= 0.475;			
$arrayTMT[16]= 0.463;
$arrayTMT[17]= 0.452;
$arrayTMT[18]= 0.442;
$arrayTMT[19]= 0.433;
$arrayTMT[20]= 0.425;
$arrayTMT[21]= 0.418;
$arrayTMT[22]= 0.411;
$arrayTMT[23]= 0.404;
$arrayTMT[24]= 0.399;
$arrayTMT[25]= 0.393;
$arrayTMT[26]= 0.388;
$arrayTMT[27]= 0.384;
$arrayTMT[28]= 0.38;
$arrayTMT[29]= 0.376;
$arrayTMT[30]= 0.372;

open(MATCHLIST, "<$fileIn_match");



while (<MATCHLIST>) { # read in an ID

	$Line= $_;				#define randomer line
	chomp $Line;
	$counts = 0;
	$samples = 0;
	@TABs = split(/\t/, $Line);		#separate line by tabs
	$randomer = @TABs[0];			#define randomer
	$n1 = @TABs[-6];				#randomer counts in conditions 1 rep 1
	$n2 = @TABs[-5];				#randomer counts in conditions 1 rep 2
	$n3 = @TABs[-4];				#randomer counts in conditions 1 rep 3

	$h1 = @TABs[-3];				#randomer counts in conditions 2 rep 1
	$h2 = @TABs[-2];				#randomer counts in conditions 2 rep 2
	$h3 = @TABs[-1];				#randomer counts in conditions 2 rep 3
	$total = $h1+$h2+$h3+$n1+$n2+$n3;

	
	if($n1>0){
		$samples++;
	}
	if($n2>0){
		$samples++;
	}
	if($n3>0){
		$samples++;
	}
	if($h1>0){
		$samples++;
	}
	if($h2>0){
		$samples++;
	}
	if($h3>0){
		$samples++;
	}
	if($samples>0){
		$average = $total/$samples;
	#loop through loci and add add randomer-counts key value pair to chromosome bin array of hashes
	
		$j=1;
		$len = @TABs;
		while ($j<$len){
			$loc= $TABs[$j];
			if (substr($loc,0,3) eq "X_F"){
				$position = substr($loc,4);
				$kilo = $position/100;
				$arrayX[$kilo]{$randomer} = $average;
				$arrayX[$kilo+1]{$randomer} = $average;
				$arrayX[$kilo+2]{$randomer} = $average;
				$arrayX[$kilo+3]{$randomer} = $average;
				$arrayX[$kilo+4]{$randomer} = $average;
			}
			if (substr($loc,0,3) eq "X_R"){
				$position = substr($loc,4);
				$kilo = $position/100;
				$arrayX[$kilo]{$randomer} = $average;
				$arrayX[$kilo-1]{$randomer} = $average;
				$arrayX[$kilo-2]{$randomer} = $average;
				$arrayX[$kilo-3]{$randomer} = $average;
				$arrayX[$kilo-4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "2L_F"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array2L[$kilo]{$randomer} = $average;
				$array2L[$kilo+1]{$randomer} = $average;
				$array2L[$kilo+2]{$randomer} = $average;
				$array2L[$kilo+3]{$randomer} = $average;
				$array2L[$kilo+4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "2L_R"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array2L[$kilo]{$randomer} = $average;
				$array2L[$kilo-1]{$randomer} = $average;
				$array2L[$kilo-2]{$randomer} = $average;
				$array2L[$kilo-3]{$randomer} = $average;
				$array2L[$kilo-4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "2R_F"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array2R[$kilo]{$randomer} = $average;
				$array2R[$kilo+1]{$randomer} = $average;
				$array2R[$kilo+2]{$randomer} = $average;
				$array2R[$kilo+3]{$randomer} = $average;
				$array2R[$kilo+4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "2R_R"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array2R[$kilo]{$randomer} = $average;
				$array2R[$kilo-1]{$randomer} = $average;
				$array2R[$kilo-2]{$randomer} = $average;
				$array2R[$kilo-3]{$randomer} = $average;
				$array2R[$kilo-4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "3L_F"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array3L[$kilo]{$randomer} = $average;
				$array3L[$kilo+1]{$randomer} = $average;
				$array3L[$kilo+2]{$randomer} = $average;
				$array3L[$kilo+3]{$randomer} = $average;
				$array3L[$kilo+4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "3L_R"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array3L[$kilo]{$randomer} = $average;
				$array3L[$kilo-1]{$randomer} = $average;
				$array3L[$kilo-2]{$randomer} = $average;
				$array3L[$kilo-3]{$randomer} = $average;
				$array3L[$kilo-4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "3R_F"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array3R[$kilo]{$randomer} = $average;
				$array3R[$kilo+1]{$randomer} = $average;
				$array3R[$kilo+2]{$randomer} = $average;
				$array3R[$kilo+3]{$randomer} = $average;
				$array3R[$kilo+4]{$randomer} = $average;
			}
			if (substr($loc,0,4) eq "3R_R"){
				$position = substr($loc,5);
				$kilo = $position/100;
				$array3R[$kilo]{$randomer} = $average;
				$array3R[$kilo-1]{$randomer} = $average;
				$array3R[$kilo-2]{$randomer} = $average;
				$array3R[$kilo-3]{$randomer} = $average;
				$array3R[$kilo-4]{$randomer} = $average;
			}
			$j=$j+2;
		}
	}
			
}

close MATCHLIST;



open(OUTPUT, ">$file_out")
	or die "Can't open input: $file $!";
	
for $i ( 0 .. $#arrayX ) {
	$highest = 0;
	$second_highest = 0;
	$lowest = 10000000;
	$n=0;

	for $randomer ( keys %{ $arrayX[$i] } ) {
		$counts = $arrayX[$i]{$randomer};
		$n++;
		if($counts<$lowest){
			$lowest=$counts;
		}
		if($counts>$second_highest){
			if($counts>$highest){
				$second_highest = $highest;
				$highest = $counts;
				$bad_randomer = $randomer;
			}
			else{
				$second_highest = $counts;
			}
		}
	}
	if ($n < 3){
		print OUTPUT "X\t$i\t$bad_randomer\t$highest\t$second_highest\n";
	}
	if ($n > 2 && $highest>$lowest ){
		$range = $highest-$lowest;
		$diff = $highest-$second_highest;
		$ratio = $diff/$range;
		$Tau = 0.372;
		if ($n<=30){
			$Tau = $arrayTMT[$n];
		}
		if ($ratio > $Tau){
			print OUTPUT "X\t$i\t$bad_randomer\t$highest\t$second_highest\n";
		}
	}
}

for $i ( 0 .. $#array2L ) {
	$highest = 0;
	$second_highest = 0;
	$lowest = 10000000;
	$n=0;

	for $randomer ( keys %{ $array2L[$i] } ) {
		$counts = $array2L[$i]{$randomer};
		$n++;
		if($counts<$lowest){
			$lowest=$counts;
		}
		if($counts>$second_highest){
			if($counts>$highest){
				$second_highest = $highest;
				$highest = $counts;
				$bad_randomer = $randomer;
			}
			else{
				$second_highest = $counts;
			}
		}
	}
	if ($n < 3){
		print OUTPUT "2L\t$i\t$bad_randomer\t$highest\t$second_highest\n";
	}
	if ($n > 2 && $highest>$lowest){
		$range = $highest-$lowest;
		$diff = $highest-$second_highest;
		$ratio = $diff/$range;
		$Tau = 0.372;
		if ($n<=30){
			$Tau = $arrayTMT[$n];
		}
		if ($ratio > $Tau){
			print OUTPUT "2L\t$i\t$bad_randomer\t$highest\t$second_highest\n";
		}
	}
}

for $i ( 0 .. $#array2R ) {
	$highest = 0;
	$second_highest = 0;
	$lowest = 10000000;
	$n=0;

	for $randomer ( keys %{ $array2R[$i] } ) {
		$counts = $array2R[$i]{$randomer};
		$n++;
		if($counts<$lowest){
			$lowest=$counts;
		}
		if($counts>$second_highest){
			if($counts>$highest){
				$second_highest = $highest;
				$highest = $counts;
				$bad_randomer = $randomer;
			}
			else{
				$second_highest = $counts;
			}
		}
	}
	if ($n < 3){
		print OUTPUT "2R\t$i\t$bad_randomer\t$highest\t$second_highest\n";
	}
	if ($n > 2 && $highest>$lowest){
		$range = $highest-$lowest;
		$diff = $highest-$second_highest;
		$ratio = $diff/$range;
		$Tau = 0.372;
		if ($n<=30){
			$Tau = $arrayTMT[$n];
		}
		if ($ratio > $Tau){
			print OUTPUT "2R\t$i\t$bad_randomer\t$highest\t$second_highest\n";
		}
	}
}



for $i ( 0 .. $#array3L ) {
	$highest = 0;
	$second_highest = 0;
	$lowest = 10000000;
	$n=0;

	for $randomer ( keys %{ $array3L[$i] } ) {
		$counts = $array3L[$i]{$randomer};
		$n++;
		if($counts<$lowest){
			$lowest=$counts;
		}
		if($counts>$second_highest){
			if($counts>$highest){
				$second_highest = $highest;
				$highest = $counts;
				$bad_randomer = $randomer;
			}
			else{
				$second_highest = $counts;
			}
		}
	}
	if ($n < 3){
		print OUTPUT "3L\t$i\t$bad_randomer\t$highest\t$second_highest\n";
	}
	if ($n > 2 && $highest>$lowest){
		$range = $highest-$lowest;
		$diff = $highest-$second_highest;
		$ratio = $diff/$range;
		$Tau = 0.372;
		if ($n<=30){
			$Tau = $arrayTMT[$n];
		}
		if ($ratio > $Tau){
			print OUTPUT "3L\t$i\t$bad_randomer\t$highest\t$second_highest\n";
		}
	}
}

for $i ( 0 .. $#array3R ) {
	$highest = 0;
	$second_highest = 0;
	$lowest = 10000000;
	$n=0;

	for $randomer ( keys %{ $array3R[$i] } ) {
		$counts = $array3R[$i]{$randomer};
		$n++;
		if($counts<$lowest){
			$lowest=$counts;
		}
		if($counts>$second_highest){
			if($counts>$highest){
				$second_highest = $highest;
				$highest = $counts;
				$bad_randomer = $randomer;
			}
			else{
				$second_highest = $counts;
			}
		}
	}
	if ($n < 3){
		print OUTPUT "3R\t$i\t$bad_randomer\t$highest\t$second_highest\n";
	}
	if ($n > 2 && $highest>$lowest){
		$range = $highest-$lowest;
		$diff = $highest-$second_highest;
		$ratio = $diff/$range;
		$Tau = 0.372;
		if ($n<=30){
			$Tau = $arrayTMT[$n];
		}
		if ($ratio > $Tau){
			print OUTPUT "3R\t$i\t$bad_randomer\t$highest\t$second_highest\n";
		}
	}
}

close OUTPUT;