#This script takes in the "randomer_genomic_weighted_match_list" file and the experimental 
#sample randomer lists for each replicate(output randomer lists from "experimental_read_miner" pipeline)
#and calculates the number of times each randomer was seen in each replicate.  An output file in which each line
#begins with a unique randomer, followed in tab-delimited format by its genomic mappings and counts in each replicate
#is created.  This file is called "randomer_genomic_weighted_match_list_with_replicate_counts" 
#and is used in the dixon outlier test.

#Arguments

$fileIn_match = $ARGV[0];		# input randomer_genomic_weighted_match_list
$fileIn_dataN1 = $ARGV[1];		# input untreated replicate one randomers
$fileIn_dataN2 = $ARGV[2];		# input untreated replicate two randomers
$fileIn_dataN3 = $ARGV[3];		# input untreated replicate three randomers
$fileIn_dataH1 = $ARGV[4];		# input treated replicate one randomers
$fileIn_dataH2 = $ARGV[5];		# input treated replicate two randomers
$fileIn_dataH3 = $ARGV[6];		# input treated replicate three randomers

#Output file
$file_out = "randomer_genomic_weighted_match_list_with_replicate_counts";	# output file with randomer-genomic mappings as well as replicate counts







open(MATCHLIST, "<$fileIn_match");



while (<MATCHLIST>) { # read in an ID

	$Line= $_;
	chop $Line;
	@TABs = split(/\t/, $Line);
	$randomer = @TABs[0];
	$N1_hash{$randomer}=0;
	$N2_hash{$randomer}=0;
	$N3_hash{$randomer}=0;
	$H1_hash{$randomer}=0;
	$H2_hash{$randomer}=0;
	$H3_hash{$randomer}=0;
		
			
}

close MATCHLIST;

open(DATA, "<$fileIn_dataN1")
	or die "Can't open output: $file_out $!";

while (<DATA>) { # read in an ID
	
	$Line= $_;
	chop $Line;
	$randomer = $Line;
	@deg_keys = &degen($randomer);
	foreach $d_key (@deg_keys) {
		if (exists $N1_hash{$d_key} ) {
			$N1_hash{$d_key}++;
			last;
		}			
	
	}	

}

close DATA;


open(DATA, "<$fileIn_dataN2")
	or die "Can't open output: $file_out $!";

while (<DATA>) { # read in an ID
	
	$Line= $_;
	chop $Line;
	$randomer = $Line;
	@deg_keys = &degen($randomer);
	foreach $d_key (@deg_keys) {
		if (exists $N2_hash{$d_key} ) {
			$N2_hash{$d_key}++;
			last;
		}			
	}	

}

close DATA;



open(DATA, "<$fileIn_dataN3")
	or die "Can't open output: $file_out $!";

while (<DATA>) { # read in an ID
	
	$Line= $_;
	chop $Line;
	$randomer = $Line;
	@deg_keys = &degen($randomer);
	foreach $d_key (@deg_keys) {
		if (exists $N3_hash{$d_key} ) {
			$N3_hash{$d_key}++;
			last;
		}			
		
	}	

}

close DATA;


open(DATA, "<$fileIn_dataH1")
	or die "Can't open output: $file_out $!";

while (<DATA>) { # read in an ID
	
	$Line= $_;
	chop $Line;
	$randomer = $Line;
	@deg_keys = &degen($randomer);
	foreach $d_key (@deg_keys) {
		if (exists $H1_hash{$d_key} ) {
			$H1_hash{$d_key}++;
			last;
		}			
	}	

}

close DATA;


open(DATA, "<$fileIn_dataH2")
	or die "Can't open output: $file_out $!";

while (<DATA>) { # read in an ID
	
	$Line= $_;
	chop $Line;
	$randomer = $Line;
	@deg_keys = &degen($randomer);
	foreach $d_key (@deg_keys) {
		if (exists $H2_hash{$d_key} ) {
			$H2_hash{$d_key}++;
			last;
		}			
	}	

}

close DATA;


open(DATA, "<$fileIn_dataH3")
	or die "Can't open output: $file_out $!";

while (<DATA>) { # read in an ID
	
	$Line= $_;
	chop $Line;
	$randomer = $Line;
	@deg_keys = &degen($randomer);
	foreach $d_key (@deg_keys) {
		if (exists $H3_hash{$d_key} ) {
			$H3_hash{$d_key}++;
			last;
		}			
	}	

}

close DATA;



open(OUTPUT, ">$file_out")
	or die "Can't open input: $file $!";



open(MATCHLIST, "<$fileIn_match")
	or die "Can't open input: $file $!";


while (<MATCHLIST>) { # read in an ID

	$Line= $_;
	chop $Line;
	@TABs = split(/\t/, $Line);
	$randomer = @TABs[0];
	print OUTPUT "$Line\tdata\t$N1_hash{$randomer}\t$N2_hash{$randomer}\t$N3_hash{$randomer}\t$H1_hash{$randomer}\t$H2_hash{$randomer}\t$H3_hash{$randomer}\n";
		
	
			
}


close MATCHLIST;




close OUTPUT;



sub degen {
	my $dna = @_[0];
 	my $len = length($dna);
 	my $letter = "";
 	my @degs = ();
 	my @unique_degs = ();
 	my $deg = "";
 	my $left_side = "";
 	my $right_side = "";
 	my %seen =() ;

	for ($letter = 0; $letter < $len; $letter++) {
		#this does one bp degenerates
		$left_side = substr($dna,0,$letter);
		$right_side = substr($dna,$letter+1);
		$deg = $left_side . "A" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "T" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "C" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "G" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "N" . $right_side;
		push @degs, $deg;
		#this does one bp insertions i.e. left side goes through letter, right side end one before end
		$left_side = substr($dna,0,$letter);
		$right_side = substr($dna,$letter, $len-$letter-1);
		$deg = $left_side . "A" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "T" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "C" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "G" . $right_side;
		push @degs, $deg;
		$deg = $left_side . "N" . $right_side;
		push @degs, $deg;
	}
	@unique_degs = grep { ! $seen{$_}++ } @degs ;	    

  return @unique_degs;
}
