#This script takes input of shuffled paired-end file containing all enhancer construct match 
#reads and quality filters based on Q_randomer, Q_dmel, and len_dmel values (50 50 50 recommended)
#then outputs those which pass


#ARGUMENTS
#0 	$file_in (string) is the paired-end fastq input file which is the output from shell_concatenate.pl 
#1  $file_out (string) is the file of paired end sequences which have been quality filtered
#2  $Q_randomer (int) is the maximum acceptable probability in percent of a single error in the randomer stretch in read 2  
#3  $Q_dmel (int) is the maximum acceptable probability in percent of a single error in the Drosophila melanogaster genomic fragment in read 1 
#4  $len_dmel (int) is the minimum length of read 1 genomic fragment

if ($#ARGV == 4) {
	$file_in = $ARGV[0];
	$file_out = $ARGV[1];
	$Q_randomer = $ARGV[2];
	$Q_dmel = $ARGV[3];
	$len_dmel = $ARGV[4];
}




open(OUTPUT, ">$file_out")
	or die "Can't open output: $file_out $!";

open(FILE, "<$file_in")
	or die "Can't open input: $file $!";

$phred = 33;


while (<FILE>) {

	$R1_ID1 = $_; chomp($R1_ID1);
	$R1_seq = <FILE>; chomp($R1_seq);
	$R1_ID2 = <FILE>; chomp($R1_ID2);
	$R1_qual = <FILE>; chomp($R1_qual);
	$R2_ID1 = <FILE>; chomp($R2_ID1);
	$R2_seq = <FILE>; chomp($R2_seq);
	$R2_ID2 = <FILE>; chomp($R2_ID2);
	$R2_qual = <FILE>; chomp($R2_qual);

	$qual_R1 = substr($R1_qual,0,$len_dmel); 
	@ASCII_R1 = unpack("C*", $qual_R1);
	$prob = 100;
	foreach $value (@ASCII_R1) {
		$value = $value - $phred;
		$prob = $prob * (1-(10**(-$value/10)));
	}
	if ($prob>=$Q_dmel){

		$qual_R2 = substr($R2_qual,0,25); 
		@ASCII_R2 = unpack("C*", $qual_R2);
		$prob = 100;
		foreach $value (@ASCII_R2) {
			$value = $value - $phred;
			$prob = $prob * (1-(10**(-$value/10)));
		}
		if ($prob>=$Q_randomer){
			print OUTPUT "$R1_ID1\n$R1_seq\n$R1_ID2\n$R1_qual\n$R2_ID1\n$R2_seq\n$R2_ID2\n$R2_qual\n";
		}
	}
}

close FILE;
close OUTPUT;
