#this shell script takes in the bowtie2 output and creates a list matching randomers to their respective genomic locations.
#The final output list file is called "randomer_genomic_weighted_match_list"


#Arguments
$bowtie_alignment_file = $ARGV[0];  # (string) input file which is the output from bowtie2 of the fasta file of randomer_genomic matches from the illumina_to_bowtie pipeline
$max_repeats = $ARGV[1];			#(int) max number of repeats allowed.  specified here by user (suggested to start with 10)

#Output files
$file_out_non_repeats = "file_out_non_repeats";	#output file of non repetitive alignments
$file_out_repeats = "file_out_repeats";		# output file of repetitive alignments
$highest_score_repeats = "highest_score_repeats";	# output file of highest score alignments
$repeats_lined = "repeats_lined";			# output file of repetitive alignemnts collapsed into a single line
$repeats_acceptable = "repeats_acceptable";		# output file of repetitive alignemnts not greater than threshold $max_repeats
$highly_repetitive = "highly_repetitive";		# output file of repetitive alignemnts greater than threshold $max_repeats
$repeats_sorted = "repeats_sorted";			# output file of repetitive alignemnts sorted by genomic position
$singletons = "singletons";				# output file of repetitive alignemnts with only one maximally scored alignment
$proper_repeats = "proper_repeats";		# output file of repetitive alignemnts below max which are not singletons
$lined_non_repeats = "lined_non_repeats";		# output file of non repetitive alignemnts collapsed into a single line
$all_not_proper_repeats = "all_not_proper_repeats"; # output file of unique alignments and highly repetitive alignments



system("perl randomer_genomic_bowtie2_alignment_repeat_non_repeat_separater.pl $bowtie_alignment_file $file_out_non_repeats $file_out_repeats");
system("perl lesser_alignments_filter.pl $file_out_repeats $highest_score_repeats");
system("perl repeat_loc_liner.pl $highest_score_repeats $repeats_lined");
system("perl repeat_region_number_limiter.pl $repeats_lined $max_repeats > $repeats_acceptable");
system("perl repeat_region_number_limiter_bad_repeats_lister.pl $repeats_lined $max_repeats > $highly_repetitive");
system("perl rep_sorter.pl $repeats_acceptable $repeats_sorted");
system("perl repeats_singleton_separater.pl $repeats_sorted $singletons $proper_repeats");
system("perl non_repeats_single_printer.pl $file_out_non_repeats $lined_non_repeats");
system("cat $lined_non_repeats $highly_repetitive $singletons > $all_not_proper_repeats");
system("perl randomer_genomic_weighted_list_maker_bowtie2_all.pl $all_not_proper_repeats $proper_repeats randomer_genomic_weighted_match_list");

#randomer_genomic_weighted_match_list is the output file which is used downstream